use crate::prelude::*;
use library_loader::load_function_ptrs;

use std::os::raw::c_void;

load_function_ptrs!(RayTracingPipelineFunctions, {
    vkCmdSetRayTracingPipelineStackSizeKHR(
        command_buffer: VkCommandBuffer,
        pipelineStackSize: u32
    ) -> (),

    vkCmdTraceRaysIndirectKHR(
        commandBuffer: VkCommandBuffer,
        pRaygenShaderBindingTable: *const VkStridedDeviceAddressRegionKHR,
        pMissShaderBindingTable:  *const VkStridedDeviceAddressRegionKHR,
        pHitShaderBindingTable:  *const VkStridedDeviceAddressRegionKHR,
        pCallableShaderBindingTable: *const VkStridedDeviceAddressRegionKHR,
        indirectDeviceAddress: VkDeviceAddress
    ) -> (),

    vkCmdTraceRaysKHR(
        commandBuffer: VkCommandBuffer,
        pRaygenShaderBindingTable: *const VkStridedDeviceAddressRegionKHR,
        pMissShaderBindingTable:  *const VkStridedDeviceAddressRegionKHR,
        pHitShaderBindingTable:  *const VkStridedDeviceAddressRegionKHR,
        pCallableShaderBindingTable: *const VkStridedDeviceAddressRegionKHR,
        width: u32,
        height: u32,
        depth: u32
    ) -> (),

    vkCreateRayTracingPipelinesKHR(
        device: VkDevice,
        deferredOperation: VkDeferredOperationKHR,
        pipelineCache: VkPipelineCache,
        createInfoCount: u32,
        pCreateInfos: *const VkRayTracingPipelineCreateInfoKHR,
        pAllocator: *const VkAllocationCallbacks,
        pPipelines: *mut VkPipeline
    ) -> VkResult,

    vkGetRayTracingCaptureReplayShaderGroupHandlesKHR(
        device: VkDevice,
        pipeline: VkPipeline,
        firstGroup: u32,
        groupCount: u32,
        dataSize: isize,
        pData: *mut c_void
    ) -> VkResult,

    vkGetRayTracingShaderGroupHandlesKHR(
        device: VkDevice,
        pipeline: VkPipeline,
        firstGroup: u32,
        groupCount: u32,
        dataSize: isize,
        pData: *mut c_void
    ) -> VkResult,

    vkGetRayTracingShaderGroupStackSizeKHR(
        device: VkDevice,
        pipeline: VkPipeline,
        group: u32,
        groupShader: VkShaderGroupShaderKHR
    ) -> VkDeviceSize,
});

impl RayTracingPipelineFunctions {
    pub fn new(instance_functions: &InstanceFunctions, device: VkDevice) -> Self {
        Self::load(|name| unsafe {
            instance_functions.vkGetDeviceProcAddr(device, name.as_ptr()) as *const std::ffi::c_void
        })
    }
}
